@@ -38,34 +38,6 @@ spark_serialize_csv_file <- function(sc, df, columns, repartition) {
   df
 }
 
-spark_serialize_typed_list <- function(sc, df, columns, repartition) {
-  structType <- spark_data_build_types(sc, columns)
-
-  # Map date and time columns as standard doubles
-  df <- as.data.frame(lapply(df, function(e) {
-    if (inherits(e, "POSIXt") || inherits(e, "Date"))
-      sapply(e, function(t) {
-        class(t) <- NULL
-        t
-      })
-    else
-      e
-  }), optional = TRUE)
-
-  rows <- lapply(seq_len(NROW(df)), function(e) as.list(df[e,]))
-
-  rdd <- invoke_static(
-    sc,
-    "sparklyr.Utils",
-    "createDataFrame",
-    spark_context(sc),
-    rows,
-    as.integer(if (repartition <= 0) 1 else repartition)
-  )
-
-  invoke(hive_context(sc), "createDataFrame", rdd, structType)
-}
-
 spark_serialize_csv_string <- function(sc, df, columns, repartition) {
   structType <- spark_data_build_types(sc, columns)
 
@@ -143,7 +115,13 @@ spark_serialize_csv_scala <- function(sc, df, columns, repartition) {
   invoke(hive_context(sc), "createDataFrame", rdd, structType)
 }
 
-spark_data_copy <- function(sc, df, name, repartition, serializer = "csv_file") {
+spark_data_copy <- function(
+  sc,
+  df,
+  name,
+  repartition,
+  serializer = getOption("sparklyr.copy.serializer", "csv_file")) {
+
   if (!is.numeric(repartition)) {
     stop("The repartition parameter must be an integer")
   }
@@ -152,13 +130,11 @@ spark_data_copy <- function(sc, df, name, repartition, serializer = "csv_file")
     stop("Using a local file to copy data is not supported for remote clusters")
   }
 
-  csv_exists <- spark_csv_is_loaded(sc)
   serializer <- ifelse(is.null(serializer),
-                       ifelse(spark_connection_is_local(sc) && csv_exists,
-                              "csv_file",
-                              ifelse(spark_connection_is_yarn_client(sc),
-                                     "csv_file_scala",
-                                     "csv_string")),
+                       ifelse(spark_connection_is_local(sc) ||
+                              spark_connection_is_yarn_client(sc),
+                              "csv_file_scala",
+                              "csv_string"),
                        serializer)
 
   # Spark unfortunately has a number of issues with '.'s in column names, e.g.
@@ -174,13 +150,14 @@ spark_data_copy <- function(sc, df, name, repartition, serializer = "csv_file")
   columns <- lapply(df, function(e) {
     if (is.factor(e))
       "character"
+    else if ("POSIXct" %in% class(e))
+      "timestamp"
     else
       typeof(e)
   })
 
   serializers <- list(
     "csv_file" = spark_serialize_csv_file,
-    "typed_list" = spark_serialize_typed_list,
     "csv_string" = spark_serialize_csv_string,
     "csv_file_scala" = spark_serialize_csv_scala
   )
